﻿#include "Track2D.h"


Track2D::Track2D():BaseTrack(8, 4)
{
    std::shared_ptr<mytracker_f::KalmanFilter_f> kf = std::make_shared<mytracker_f::KalmanFilter_f>(8, 4);
    kf->F_ <<
        1, 0, 0, 0, 1, 0, 0, 0,
        0, 1, 0, 0, 0, 1, 0, 0,
        0, 0, 1, 0, 0, 0, 1, 0,
        0, 0, 0, 1, 0, 0, 0, 1,
        0, 0, 0, 0, 1, 0, 0, 0,
        0, 0, 0, 0, 0, 1, 0, 0,
        0, 0, 0, 0, 0, 0, 1, 0,
        0, 0, 0, 0, 0, 0, 0, 1;
    kf->P_ <<
        10, 0, 0, 0, 0, 0, 0, 0,
        0, 10, 0, 0, 0, 0, 0, 0,
        0, 0, 10, 0, 0, 0, 0, 0,
        0, 0, 0, 10, 0, 0, 0, 0,
        0, 0, 0, 0, 10000, 0, 0, 0,
        0, 0, 0, 0, 0, 10000, 0, 0,
        0, 0, 0, 0, 0, 0, 10000, 0,
        0, 0, 0, 0, 0, 0, 0, 10000;
    kf->H_ <<
        1, 0, 0, 0, 0, 0, 0, 0,
        0, 1, 0, 0, 0, 0, 0, 0,
        0, 0, 1, 0, 0, 0, 0, 0,
        0, 0, 0, 1, 0, 0, 0, 0;
    kf->Q_ <<
        1, 0, 0, 0, 0, 0, 0, 0,
        0, 1, 0, 0, 0, 0, 0, 0,
        0, 0, 1, 0, 0, 0, 0, 0,
        0, 0, 0, 1, 0, 0, 0, 0,
        0, 0, 0, 0, 0.01, 0, 0, 0,
        0, 0, 0, 0, 0, 0.01, 0, 0,
        0, 0, 0, 0, 0, 0, 0.0001, 0,
        0, 0, 0, 0, 0, 0, 0, 0.0001;
    kf->R_ <<
        1, 0, 0, 0,
        0, 1, 0, 0,
        0, 0, 10, 0,
        0, 0, 0, 10;

    kf_ = kf;
}

void Track2D::Init(const std::vector<float>& data)
{
    if (data.size() != 5)
        return;
    std::vector<float> tmp(data.begin() + 1, data.end());
    BaseTrack::Init(tmp);
}
void Track2D::Update(const std::vector<float>& data)
{
    std::vector<float> out(data.begin() + 1, data.end());
    BaseTrack::Update(out);
}

double Track2D::CalculateIou(const std::vector<float>& data)
{
    if (data.size() != 5)
        return 0;
    int input_type = data[0];
    int obj_type = m_obj.type;
    if (input_type != obj_type)
        return 0.0f;
    std::vector<float> states;
    GetStateData(states);

    //auto xx1 = std::max(states[0], data[0]);
    //auto yy1 = std::max(states[1], data[1]);
    //auto xx2 = std::min(det.br().x, trk.br().x);
    //auto yy2 = std::min(det.br().y, trk.br().y);
    //auto w = std::max(0, xx2 - xx1);
    //auto h = std::max(0, yy2 - yy1);

    //// calculate area of intersection and union
    //float det_area = det.area();
    //float trk_area = trk.area();
    //auto intersection_area = w * h;
    //float union_area = det_area + trk_area - intersection_area;
    //auto iou = intersection_area / union_area;

    auto w = std::max(std::min((states[0] + states[2]), (data[1] + data[3])) - std::max(states[0], data[1]), 0.0f);
    auto h = std::max(std::min((states[1] + states[3]), (data[2] + data[4])) - std::max(states[1], data[2]), 0.0f);

    auto intersection_area = w * h;
    float union_area = states[2] * states[3] + data[3] * data[4] - intersection_area;
    auto iou = intersection_area / union_area;
	return iou;
}
